/*
 * Copyright 2016 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file delete.c
 * Implementation of the s3 DSI delete
 */
#include "globus_gridftp_server_s3_base.h"

static
void *
globus_l_s3_delete_thread(
    void                               *user_arg);

static
void
globus_l_s3_delete_complete(
    S3Status                            status,
    const S3ErrorDetails               *errorDetails,
    void                               *callbackData);

static
S3Status
globus_l_s3_delete_properties(
    const S3ResponseProperties         *properties,
    void                               *callback_arg);

typedef struct globus_l_s3_delete_s
{
    /** GridFTP delete operation */
    globus_gfs_operation_t              op;
    /** Bucket Context derived from the session info and the delete pathname */
    S3BucketContext                     bucket_context;
    /** pointer into pathname that contains the bucket name */
    char                               *bucket_name;
    /** pointer into pathname that contains the key name */
    char                               *key;
    /** number of retries attempted */
    int                                 retries;
    /** S3's status of the operation */
    S3Status                            status;
    /** Result of the operation */
    globus_result_t                     result;
    /** Copy of the pathname string split into bucket_name and key */
    char                                pathname[];
}
globus_l_s3_delete_t;

/**
 * @brief delete
 * @details
 */
void
globus_i_s3_delete(
    globus_gfs_operation_t              op,
    globus_gfs_command_info_t          *cmd_info,
    void                               *user_arg)
{
    globus_i_s3_handle_t               *s3_handle = user_arg;
    globus_thread_t                     t;
    globus_l_s3_delete_t               *delete_state = NULL;
    int                                 rc;
    globus_result_t                     result = GLOBUS_SUCCESS;

    GlobusS3Enter();

    GlobusS3Info("delete: op=%p pathname=%s\n",
                (void *) op,
                cmd_info->pathname);

    assert(s3_handle != NULL);

    delete_state = malloc(sizeof(globus_l_s3_delete_t) + strlen(cmd_info->pathname) + 1);

    if (delete_state == NULL)
    {
        result = GlobusGFSErrorMemory("delete_state");
        goto failure;
    }

    *delete_state = (globus_l_s3_delete_t)
    {
        .op                           = op
    };

    strcpy(delete_state->pathname, cmd_info->pathname);
    globus_i_s3_path_parse(delete_state->pathname, &delete_state->bucket_name, &delete_state->key);

    if (delete_state->key == NULL)
    {
        result = GlobusGFSErrorSystemError("delete", EINVAL);
        goto failure;
    }

    delete_state->bucket_context = GLOBUS_S3_BUCKET_CONTEXT(s3_handle, delete_state->bucket_name);

    rc = globus_thread_create(
            &t,
            NULL,
            globus_l_s3_delete_thread,
            delete_state);

    if (rc != GLOBUS_SUCCESS)
    {
        result = GlobusGFSErrorGeneric("globus_thread_create");
    }

failure:
    if (result != GLOBUS_SUCCESS)
    {
        GlobusS3Info("finished_command: op=%p result=%#x path=%s\n",
                    (void *) op,
                    result,
                    cmd_info->pathname);
        globus_gridftp_server_finished_command(op, result, NULL);

        free(delete_state);
    }

    GlobusS3Exit();

    return;
}
/* globus_i_s3_delete() */

static
void *
globus_l_s3_delete_thread(
    void                               *user_arg)
{
    GlobusS3Enter();

    globus_l_s3_delete_t               *delete_state = user_arg;

    assert(delete_state->key != NULL);
    do
    {
        delete_state->status = S3StatusOK;
        delete_state->result = GLOBUS_SUCCESS;

        globus_i_s3_retry_delay(delete_state->retries);

        GlobusS3Info("REST: op=%p S3_delete_object(bucket=%s, key=%s) try=%d\n",
                    (void *) delete_state->op,
                    delete_state->bucket_name,
                    delete_state->key,
                    delete_state->retries);

        S3_delete_object(
                &delete_state->bucket_context,
                delete_state->key,
                NULL,
                &(S3ResponseHandler)
                {
                    .propertiesCallback   = globus_l_s3_delete_properties,
                    .completeCallback     = globus_l_s3_delete_complete
                },
                delete_state);

        GlobusS3DebugOpStatus(S3_delete_object, delete_state);
    } while (S3_status_is_retryable(delete_state->status)
            && (delete_state->retries++) < globus_i_s3_max_retries);

    GlobusS3Info("finished_command: op=%p result=%#x path=%s\n",
                (void *) delete_state->op,
                (unsigned int) delete_state->result,
                delete_state->pathname);

    globus_gridftp_server_finished_command(delete_state->op, delete_state->result, NULL);
    free(delete_state);

    return NULL;
}
/* globus_l_s3_delete_thread() */

static
S3Status
globus_l_s3_delete_properties(
    const S3ResponseProperties         *properties,
    void                               *callback_arg)
{
    GlobusS3Enter();
    /* Don't care */
    GlobusS3Exit();
    return S3StatusOK;
}
/* globus_l_s3_delete_properties() */

static
void
globus_l_s3_delete_complete(
    S3Status                            status,
    const S3ErrorDetails               *error_details,
    void                               *callback_data)
{
    globus_l_s3_delete_t               *delete_state = callback_data;

    GlobusS3Enter();

    if (status != S3StatusOK)
    {
        char                            path[strlen(delete_state->key) + strlen(delete_state->bucket_name) + 2];

        snprintf(path, sizeof(path), "%s/%s", delete_state->bucket_name, delete_state->key);

        delete_state->result = GlobusGFSErrorS3(status, path, error_details);
    }
    GlobusS3Exit();
}
/* globus_l_s3_delete_complete() */
// vim: ft=c:
