/*
 * Copyright 2016 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file stat.c
 * Implementation of the s3 DSI stat
 */

#include "globus_gridftp_server_s3_stat.h"

static
S3Status
globus_l_s3_stat_bucket_properties(
    const S3ResponseProperties         *properties,
    void                               *callback_arg);

static
void
globus_l_s3_stat_bucket_complete(
    S3Status                            status,
    const S3ErrorDetails               *errorDetails,
    void                               *callbackData);

static
S3Status
globus_l_s3_stat_bucket_dir_callback(
    int                                 is_truncated,
    const char                         *next_marker,
    int                                 contents_count, 
    const S3ListBucketContent          *contents,
    int                                 common_prefixes_count,
    const char                        **common_prefixes,
    void                               *callback_data);

void
globus_i_s3_stat_bucket(
    globus_i_s3_stat_t                 *stat_state)
{
    stat_state->retries = 0;

    do
    {
        stat_state->status = S3StatusOK;
        stat_state->result = GLOBUS_SUCCESS;

        globus_i_s3_retry_delay(stat_state->retries);

        GlobusS3Info("REST: op=%p S3_list_bucket(bucket=%s) try=%d\n",
                    (void *) stat_state->op,
                    stat_state->bucket_name,
                    stat_state->retries);

        S3_list_bucket(
            &stat_state->bucket_context,
            NULL,
            NULL,
            "/",
            1,
            NULL,
            &(S3ListBucketHandler)
            {
                .responseHandler = (S3ResponseHandler)
                {
                    .propertiesCallback
                                  = globus_l_s3_stat_bucket_properties,
                    .completeCallback
                                  = globus_l_s3_stat_bucket_complete,
                },
                .listBucketCallback
                                  = globus_l_s3_stat_bucket_dir_callback,
            },
            stat_state);

        GlobusS3DebugOpStatus(S3_test_bucket, stat_state);
    }
    while (S3_status_is_retryable(stat_state->status)
        && (stat_state->retries++) < globus_i_s3_max_retries);

    if (stat_state->result == GLOBUS_SUCCESS)
    {
        int dev, ino;

        dev = globus_hashtable_string_hash(
                (void *) stat_state->bucket_name,
                INT_MAX);

        ino = globus_hashtable_string_hash(
                (void *) stat_state->bucket_name,
                INT_MAX);

        globus_i_s3_stat_add(
                stat_state,
                &(globus_gfs_stat_t)
                {
                    .name         = ".",
                    .mode         = S_IRWXU|S_IFDIR,
                    .nlink        = 1,
                    .dev          = dev,
                    .ino          = ino,
                    .mtime        = (time_t) -1,
                    .atime        = (time_t) -1,
                    .ctime        = (time_t) -1
                });
    }

    return;
}
/* globus_l_s3_stat_bucket() */

static
S3Status
globus_l_s3_stat_bucket_properties(
    const S3ResponseProperties         *properties,
    void                               *callback_arg)
{
    S3Status                            status = S3StatusOK;

    GlobusS3Enter();

    GlobusS3ExitStatus(status);
    return status;
}
/* globus_l_s3_stat_bucket_properties() */

static
void
globus_l_s3_stat_bucket_complete(
    S3Status                            status,
    const S3ErrorDetails               *error_details,
    void                               *callback_data)
{
    globus_i_s3_stat_t                 *stat_state = callback_data;

    GlobusS3Enter();

    if (status != S3StatusOK && stat_state->result == GLOBUS_SUCCESS)
    {
        stat_state->status = status;
        stat_state->result = GlobusGFSErrorS3(
                    status,
                    stat_state->stat_info->pathname,
                    error_details);
    }

    GlobusS3Exit();

    return;
}
/* globus_l_s3_stat_bucket_complete() */

static
S3Status
globus_l_s3_stat_bucket_dir_callback(
    int                                 is_truncated,
    const char                         *next_marker,
    int                                 contents_count, 
    const S3ListBucketContent          *contents,
    int                                 common_prefixes_count,
    const char                        **common_prefixes,
    void                               *callback_data)
{
    S3Status                            status = S3StatusOK;

    GlobusS3Enter();
    GlobusS3ExitStatus(status);

    return status;
}
/* globus_l_s3_stat_bucket_dir_callback() */

// vim: ft=c:
